@@ -66,7 +66,8 @@ struct _PeasGtkPluginManagerViewPrivate {
 enum {
   PROP_0,
   PROP_ENGINE,
-  PROP_SHOW_BUILTIN
+  PROP_SHOW_BUILTIN,
+  N_PROPERTIES
 };
 
 /* Signals */
@@ -76,6 +77,7 @@ enum {
 };
 
 static guint signals[LAST_SIGNAL];
+static GParamSpec *properties[N_PROPERTIES] = { NULL };
 
 G_DEFINE_TYPE (PeasGtkPluginManagerView, peas_gtk_plugin_manager_view, GTK_TYPE_TREE_VIEW);
 
@@ -592,6 +594,7 @@ peas_gtk_plugin_manager_view_query_tooltip (GtkWidget  *widget,
   gboolean is_row;
   GtkTreeIter iter;
   PeasPluginInfo *info;
+  gchar *message;
   GError *error = NULL;
 
   is_row = gtk_tree_view_get_tooltip_context (GTK_TREE_VIEW (widget),
@@ -600,31 +603,33 @@ peas_gtk_plugin_manager_view_query_tooltip (GtkWidget  *widget,
 
   if (!is_row)
     return FALSE;
-    
+
   convert_iter_to_child_iter (view, &iter);
 
   info = peas_gtk_plugin_manager_store_get_plugin (view->priv->store, &iter);
 
-  if (!peas_plugin_info_is_available (info, &error))
+  if (peas_plugin_info_is_available (info, &error))
+    {
+      gtk_tree_model_get (GTK_TREE_MODEL (view->priv->store), &iter,
+        PEAS_GTK_PLUGIN_MANAGER_STORE_INFO_COLUMN, &message,
+        -1);
+    }
+  else
     {
-      gchar *message;
-
       message = g_markup_printf_escaped (_("<b>The plugin '%s' could not be "
                                            "loaded</b>\nAn error occurred: %s"),
                                          peas_plugin_info_get_name (info),
                                          error->message);
-
-      gtk_tooltip_set_markup (tooltip, message);
-
-      g_free (message);
       g_error_free (error);
-
-      return TRUE;
     }
 
-  return FALSE;
+  gtk_tooltip_set_markup (tooltip, message);
+
+  g_free (message);
+
+  return TRUE;
 }
-  
+
 
 static void
 peas_gtk_plugin_manager_view_row_activated (GtkTreeView       *tree_view,
@@ -710,8 +715,7 @@ peas_gtk_plugin_manager_view_constructed (GObject *object)
                     G_CALLBACK (plugin_list_changed_cb),
                     view);
 
-  if (G_OBJECT_CLASS (peas_gtk_plugin_manager_view_parent_class)->constructed != NULL)
-    G_OBJECT_CLASS (peas_gtk_plugin_manager_view_parent_class)->constructed (object);
+  G_OBJECT_CLASS (peas_gtk_plugin_manager_view_parent_class)->constructed (object);
 }
 
 static void
@@ -725,22 +729,16 @@ peas_gtk_plugin_manager_view_dispose (GObject *object)
       view->priv->popup_menu = NULL;
     }
 
-  if (view->priv->store != NULL)
-    {
-      g_object_unref (view->priv->store);
-      view->priv->store = NULL;
-    }
-
   if (view->priv->engine != NULL)
     {
       g_signal_handlers_disconnect_by_func (view->priv->engine,
                                             plugin_list_changed_cb,
                                             view);
-
-      g_object_unref (view->priv->engine);
-      view->priv->engine = NULL;
+      g_clear_object (&view->priv->engine);
     }
 
+  g_clear_object (&view->priv->store);
+
   G_OBJECT_CLASS (peas_gtk_plugin_manager_view_parent_class)->dispose (object);
 }
 
@@ -768,39 +766,37 @@ peas_gtk_plugin_manager_view_class_init (PeasGtkPluginManagerViewClass *klass)
    *
    * The #PeasEngine this view is attached to.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_ENGINE,
-                                   g_param_spec_object ("engine",
-                                                        "engine",
-                                                        "The PeasEngine this view is attached to",
-                                                        PEAS_TYPE_ENGINE,
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_STRINGS));
+  properties[PROP_ENGINE] =
+    g_param_spec_object ("engine",
+                         "engine",
+                         "The PeasEngine this view is attached to",
+                         PEAS_TYPE_ENGINE,
+                         G_PARAM_READWRITE |
+                         G_PARAM_CONSTRUCT_ONLY |
+                         G_PARAM_STATIC_STRINGS);
 
   /**
    * PeasGtkPluginManagerView:show-builtin:
    *
    * If builtin plugins should be shown.
+   *
+   * Deprecated: 1.2: Use hidden plugins instead.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_SHOW_BUILTIN,
-                                   g_param_spec_boolean ("show-builtin",
-                                                         "show-builtin",
-                                                         "If builtin plugins should be shown",
-                                                         FALSE,
-#if GLIB_CHECK_VERSION (2, 26, 0)
-                                                         G_PARAM_DEPRECATED |
-#endif
-                                                         G_PARAM_READWRITE |
-                                                         G_PARAM_STATIC_STRINGS));
+  properties[PROP_SHOW_BUILTIN] =
+    g_param_spec_boolean ("show-builtin",
+                          "show-builtin",
+                          "If builtin plugins should be shown",
+                          FALSE,
+                          G_PARAM_DEPRECATED |
+                          G_PARAM_READWRITE |
+                          G_PARAM_STATIC_STRINGS);
 
   /**
    * PeasGtkPluginManagerView::populate-popup:
    * @view: A #PeasGtkPluginManagerView.
    * @menu: A #GtkMenu.
    *
-   * The populate-popup signal is emitted before showing the context
+   * The ::populate-popup signal is emitted before showing the context
    * menu of the view. If you need to add items to the context menu,
    * connect to this signal and add your menuitems to the @menu.
    */
@@ -815,6 +811,7 @@ peas_gtk_plugin_manager_view_class_init (PeasGtkPluginManagerViewClass *klass)
                   1,
                   GTK_TYPE_MENU);
 
+  g_object_class_install_properties (object_class, N_PROPERTIES, properties);
   g_type_class_add_private (object_class, sizeof (PeasGtkPluginManagerViewPrivate));
 }
 
@@ -845,7 +842,7 @@ peas_gtk_plugin_manager_view_new (PeasEngine *engine)
  *
  * Sets if builtin plugins should be shown.
  *
- * Deprecated: 1.2. Use hidden plugins instead.
+ * Deprecated: 1.2: Use hidden plugins instead.
  */
 void
 peas_gtk_plugin_manager_view_set_show_builtin (PeasGtkPluginManagerView *view,
@@ -896,7 +893,8 @@ peas_gtk_plugin_manager_view_set_show_builtin (PeasGtkPluginManagerView *view,
   if (iter_set && convert_child_iter_to_iter (view, &iter))
     gtk_tree_selection_select_iter (selection, &iter);
 
-  g_object_notify (G_OBJECT (view), "show-builtin");
+  g_object_notify_by_pspec (G_OBJECT (view),
+                            properties[PROP_SHOW_BUILTIN]);
 }
 
 /**
@@ -907,7 +905,7 @@ peas_gtk_plugin_manager_view_set_show_builtin (PeasGtkPluginManagerView *view,
  *
  * Returns: if builtin plugins should be shown.
  *
- * Deprecated: 1.2. Use hidden plugins instead.
+ * Deprecated: 1.2: Use hidden plugins instead.
  */
 gboolean
 peas_gtk_plugin_manager_view_get_show_builtin (PeasGtkPluginManagerView *view)
@@ -950,7 +948,7 @@ peas_gtk_plugin_manager_view_set_selected_plugin (PeasGtkPluginManagerView *view
  *
  * Returns the currently selected plugin, or %NULL if a plugin is not selected.
  *
- * Returns: the selected plugin.
+ * Returns: (transfer none): the selected plugin.
  */
 PeasPluginInfo *
 peas_gtk_plugin_manager_view_get_selected_plugin (PeasGtkPluginManagerView *view)
