@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,6 +31,7 @@
 #include "peas-engine-priv.h"
 #include "peas-plugin-info-priv.h"
 #include "peas-plugin-loader.h"
+#include "peas-plugin-loader-c.h"
 #include "peas-object-module.h"
 #include "peas-extension.h"
 #include "peas-dirs.h"
@@ -69,15 +70,17 @@ enum {
   LAST_SIGNAL
 };
 
-static guint signals[LAST_SIGNAL];
-
 /* Properties */
 enum {
   PROP_0,
   PROP_PLUGIN_LIST,
-  PROP_LOADED_PLUGINS
+  PROP_LOADED_PLUGINS,
+  N_PROPERTIES
 };
 
+static guint signals[LAST_SIGNAL];
+static GParamSpec *properties[N_PROPERTIES] = { NULL };
+
 typedef struct _LoaderInfo LoaderInfo;
 
 struct _LoaderInfo {
@@ -132,7 +135,8 @@ load_plugin_info (PeasEngine  *engine,
       engine->priv->plugin_list = g_list_prepend (engine->priv->plugin_list,
                                                   info);
 
-      g_object_notify (G_OBJECT (engine), "plugin-list");
+      g_object_notify_by_pspec (G_OBJECT (engine),
+                                properties[PROP_PLUGIN_LIST]);
     }
 }
 
@@ -431,8 +435,8 @@ peas_engine_finalize (GObject *object)
   GList *item;
 
   /* free the infos */
-  for (item = engine->priv->plugin_list; item; item = item->next)
-    _peas_plugin_info_unref (PEAS_PLUGIN_INFO (item->data));
+  g_list_free_full (engine->priv->plugin_list,
+                    (GDestroyNotify) _peas_plugin_info_unref);
 
   /* free the search path list */
   for (item = engine->priv->search_paths; item; item = item->next)
@@ -444,7 +448,6 @@ peas_engine_finalize (GObject *object)
       g_slice_free (SearchPath, sp);
     }
 
-  g_list_free (engine->priv->plugin_list);
   g_list_free (engine->priv->search_paths);
 
   G_OBJECT_CLASS (peas_engine_parent_class)->finalize (object);
@@ -475,13 +478,12 @@ peas_engine_class_init (PeasEngineClass *klass)
    * Note that the list belongs to the engine and should not be modified
    * or freed.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_PLUGIN_LIST,
-                                   g_param_spec_pointer ("plugin-list",
-                                                         "Plugin list",
-                                                         "The list of found plugins",
-                                                         G_PARAM_READABLE |
-                                                         G_PARAM_STATIC_STRINGS));
+  properties[PROP_PLUGIN_LIST] =
+    g_param_spec_pointer ("plugin-list",
+                          "Plugin list",
+                          "The list of found plugins",
+                          G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS);
 
   /**
    * PeasEngine:loaded-plugins:
@@ -503,14 +505,13 @@ peas_engine_class_init (PeasEngineClass *klass)
    *
    * Note: notify will not be called when the engine is being destroyed.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_LOADED_PLUGINS,
-                                   g_param_spec_boxed ("loaded-plugins",
-                                                       "Loaded plugins",
-                                                       "The list of loaded plugins",
-                                                       G_TYPE_STRV,
-                                                       G_PARAM_READWRITE |
-                                                       G_PARAM_STATIC_STRINGS));
+  properties[PROP_LOADED_PLUGINS] =
+    g_param_spec_boxed ("loaded-plugins",
+                        "Loaded plugins",
+                        "The list of loaded plugins",
+                        G_TYPE_STRV,
+                        G_PARAM_READWRITE |
+                        G_PARAM_STATIC_STRINGS);
 
   /**
    * PeasEngine::load-plugin:
@@ -561,6 +562,7 @@ peas_engine_class_init (PeasEngineClass *klass)
                   1, PEAS_TYPE_PLUGIN_INFO |
                   G_SIGNAL_TYPE_STATIC_SCOPE);
 
+  g_object_class_install_properties (object_class, N_PROPERTIES, properties);
   g_type_class_add_private (klass, sizeof (PeasEnginePrivate));
 
   /* We are doing some global initialization here as there is currently no
@@ -578,39 +580,17 @@ peas_engine_class_init (PeasEngineClass *klass)
 }
 
 static PeasObjectModule *
-try_to_open_loader_module (const gchar *loader_id,
-                           gboolean     in_subdir)
+load_module (const gchar *module_name,
+             const gchar *module_dir)
 {
-  gchar *tmp_dirname;
-  gchar *loader_dirname;
-  gchar *loader_basename;
   PeasObjectModule *module;
 
-  if (in_subdir)
-    {
-      tmp_dirname = peas_dirs_get_plugin_loaders_dir ();
-      loader_dirname = g_build_filename (tmp_dirname, loader_id, NULL);
-      g_free (tmp_dirname);
-    }
-  else
-    {
-      loader_dirname = peas_dirs_get_plugin_loaders_dir ();
-    }
-
-  /* Let's build the expected filename of the requested plugin loader */
-  loader_basename = g_strdup_printf ("lib%sloader.%s", loader_id, G_MODULE_SUFFIX);
-
-  g_debug ("Loading loader '%s': '%s/%s'", loader_id, loader_dirname, loader_basename);
-
-  module = peas_object_module_new (loader_basename, loader_dirname, TRUE);
-
-  g_free (loader_basename);
-  g_free (loader_dirname);
+  module = peas_object_module_new (module_name, module_dir, TRUE);
 
   if (!g_type_module_use (G_TYPE_MODULE (module)))
     {
       g_object_unref (module);
-      module = NULL;
+      return NULL;
     }
 
   return module;
@@ -621,7 +601,7 @@ get_plugin_loader (PeasEngine     *engine,
                    PeasPluginInfo *info)
 {
   LoaderInfo *loader_info;
-  gchar *loader_id;
+  gchar *loader_id, *module_name, *module_dir;
 
   loader_info = (LoaderInfo *) g_hash_table_lookup (loaders, info->loader);
 
@@ -633,19 +613,38 @@ get_plugin_loader (PeasEngine     *engine,
   if (loader_info->loader != NULL)
     return loader_info->loader;
 
-  /* We need to ensure we use the lowercase loader_id */
+  /* Create the default C plugin loader. */
+  if (g_ascii_strcasecmp (info->loader, "C") == 0)
+    {
+      loader_info->loader = peas_plugin_loader_c_new ();
+      return loader_info->loader;
+    }
+
   loader_id = g_ascii_strdown (info->loader, -1);
+  module_name = g_strconcat (loader_id, "loader", NULL);
+  module_dir = peas_dirs_get_plugin_loaders_dir ();
 
-  loader_info->module = try_to_open_loader_module (loader_id, FALSE);
-  if (loader_info->module == NULL)
-    loader_info->module = try_to_open_loader_module (loader_id, TRUE);
-
-  g_free (loader_id);
+  loader_info->module = load_module (module_name, module_dir);
 
   if (loader_info->module == NULL)
     {
-      g_hash_table_insert (loaders, g_strdup (info->loader), NULL);
-      return NULL;
+      gchar *tmp = module_dir;
+
+      module_dir = g_build_filename (module_dir, loader_id, NULL);
+      loader_info->module = load_module (module_name, module_dir);
+
+      g_free (tmp);
+
+      if (loader_info->module == NULL)
+        {
+          g_warning ("Could not load plugin loader '%s'", info->loader);
+
+          g_free (module_dir);
+          g_free (module_name);
+          g_free (loader_id);
+          g_hash_table_insert (loaders, g_strdup (info->loader), NULL);
+          return NULL;
+        }
     }
 
   loader_info->loader = PEAS_PLUGIN_LOADER (
@@ -654,17 +653,17 @@ get_plugin_loader (PeasEngine     *engine,
                                           0, NULL));
 
   g_type_module_unuse (G_TYPE_MODULE (loader_info->module));
+  g_free (module_dir);
+  g_free (module_name);
+  g_free (loader_id);
 
   if (loader_info->loader == NULL ||
-      !PEAS_IS_PLUGIN_LOADER (loader_info->loader))
+      !peas_plugin_loader_initialize (loader_info->loader))
     {
       g_warning ("Loader '%s' is not a valid PeasPluginLoader instance",
                  info->loader);
 
-      if (G_IS_OBJECT (loader_info->loader))
-        g_object_unref (loader_info->loader);
-
-      g_object_unref (loader_info->module);
+      /* This will cause the loader to be unreffed if it exists */
       g_hash_table_insert (loaders, g_strdup (info->loader), NULL);
       return NULL;
     }
@@ -699,7 +698,9 @@ peas_engine_enable_loader (PeasEngine  *engine,
   if (g_hash_table_lookup_extended (loaders, loader_id, NULL, NULL))
     return;
 
-  /* The loader is loaded in get_plugin_loader() */
+  /* We do not load the plugin loader immediately and instead
+   * load it in get_plugin_loader() so that it is loaded lazily.
+   */
   g_hash_table_insert (loaders, g_strdup (loader_id), g_new0 (LoaderInfo, 1));
 }
 
@@ -736,7 +737,7 @@ compare_plugin_info_and_name (PeasPluginInfo *info,
  * Gets the #PeasPluginInfo corresponding with @plugin_name,
  * or %NULL if @plugin_name was not found.
  *
- * Returns: the #PeasPluginInfo corresponding with
+ * Returns: (transfer none): the #PeasPluginInfo corresponding with
  * a given plugin module name.
  */
 PeasPluginInfo *
@@ -843,7 +844,8 @@ peas_engine_load_plugin_real (PeasEngine     *engine,
                               PeasPluginInfo *info)
 {
   if (load_plugin (engine, info))
-    g_object_notify (G_OBJECT (engine), "loaded-plugins");
+    g_object_notify_by_pspec (G_OBJECT (engine),
+                              properties[PROP_LOADED_PLUGINS]);
 }
 
 /**
@@ -913,7 +915,8 @@ peas_engine_unload_plugin_real (PeasEngine     *engine,
   g_debug ("Unloaded plugin '%s'", peas_plugin_info_get_module_name (info));
 
   if (!engine->priv->in_dispose)
-    g_object_notify (G_OBJECT (engine), "loaded-plugins");
+    g_object_notify_by_pspec (G_OBJECT (engine),
+                              properties[PROP_LOADED_PLUGINS]);
 }
 
 /**
@@ -1010,7 +1013,7 @@ peas_engine_create_extensionv (PeasEngine     *engine,
   extension = peas_plugin_loader_create_extension (loader, info, extension_type,
                                                    n_parameters, parameters);
 
-  if (!PEAS_IS_EXTENSION (extension))
+  if (!G_TYPE_CHECK_INSTANCE_TYPE (extension, extension_type))
     {
       g_warning ("Plugin '%s' does not provide a '%s' extension",
                  peas_plugin_info_get_module_name (info),
