@@ -60,6 +60,10 @@
  * ]|
  **/
 
+G_DEFINE_BOXED_TYPE (PeasPluginInfo, peas_plugin_info,
+                     _peas_plugin_info_ref,
+                     _peas_plugin_info_unref)
+
 PeasPluginInfo *
 _peas_plugin_info_ref (PeasPluginInfo *info)
 {
@@ -75,6 +79,8 @@ _peas_plugin_info_unref (PeasPluginInfo *info)
 
   g_free (info->module_dir);
   g_free (info->data_dir);
+  if (info->schema_source != NULL)
+    g_settings_schema_source_unref (info->schema_source);
   g_free (info->module_name);
   g_strfreev (info->dependencies);
   g_free (info->name);
@@ -92,18 +98,6 @@ _peas_plugin_info_unref (PeasPluginInfo *info)
   g_free (info);
 }
 
-GType
-peas_plugin_info_get_type (void)
-{
-  static GType the_type = 0;
-
-  if (G_UNLIKELY (!the_type))
-    the_type = g_boxed_type_register_static (g_intern_static_string ("PeasPluginInfo"),
-                                             (GBoxedCopyFunc) _peas_plugin_info_ref,
-                                             (GBoxedFreeFunc) _peas_plugin_info_unref);
-
-  return the_type;
-}
 
 GQuark
 peas_plugin_info_error_quark (void)
@@ -135,6 +129,7 @@ _peas_plugin_info_new (const gchar *filename,
   PeasPluginInfo *info;
   GKeyFile *plugin_file = NULL;
   gchar *str;
+  gchar **strv;
   gboolean b;
   GError *error = NULL;
 
@@ -213,9 +208,14 @@ _peas_plugin_info_new (const gchar *filename,
     info->authors = g_new0 (gchar *, 1);
 
   /* Get Copyright */
-  str = g_key_file_get_string (plugin_file, "Plugin", "Copyright", NULL);
-  if (str)
-    info->copyright = str;
+  strv = g_key_file_get_string_list (plugin_file, "Plugin",
+                                     "Copyright", NULL, NULL);
+  if (strv)
+    {
+      info->copyright = g_strjoinv ("\n", strv);
+
+      g_strfreev (strv);
+    }
 
   /* Get Website */
   str = g_key_file_get_string (plugin_file, "Plugin", "Website", NULL);
@@ -428,6 +428,79 @@ peas_plugin_info_get_data_dir (const PeasPluginInfo *info)
   return info->data_dir;
 }
 
+/**
+ * peas_plugin_info_get_settings:
+ * @info: A #PeasPluginInfo.
+ * @schema_id: (allow-none): The schema id.
+ *
+ * Creates a new #GSettings for the given @schema_id and if
+ * gschemas.compiled is not in the module directory an attempt
+ * will be made to create it.
+ *
+ * Returns: (transfer full): a new #GSettings, or %NULL.
+ */
+GSettings *
+peas_plugin_info_get_settings (const PeasPluginInfo *info,
+                               const gchar          *schema_id)
+{
+  GSettingsSchema *schema;
+  GSettings *settings;
+
+  g_return_val_if_fail (info != NULL, NULL);
+
+  if (info->schema_source == NULL)
+    {
+      GFile *module_dir_location;
+      GFile *gschema_compiled;
+      GSettingsSchemaSource *default_source;
+
+      module_dir_location = g_file_new_for_path (info->module_dir);
+      gschema_compiled = g_file_get_child (module_dir_location,
+                                           "gschemas.compiled");
+
+      if (!g_file_query_exists (gschema_compiled, NULL))
+        {
+          const gchar *argv[] = {
+            "glib-compile-schemas",
+            "--targetdir", info->module_dir,
+            info->module_dir,
+            NULL
+          };
+
+          g_spawn_sync (NULL, (gchar **) argv, NULL, G_SPAWN_SEARCH_PATH,
+                        NULL, NULL, NULL, NULL, NULL, NULL);
+        }
+
+      g_object_unref (gschema_compiled);
+      g_object_unref (module_dir_location);
+
+      default_source = g_settings_schema_source_get_default ();
+      ((PeasPluginInfo *) info)->schema_source =
+            g_settings_schema_source_new_from_directory (info->module_dir,
+                                                         default_source,
+                                                         FALSE, NULL);
+
+      /* glib-compile-schemas already outputted a message */
+      if (info->schema_source == NULL)
+        return NULL;
+    }
+
+  if (schema_id == NULL)
+    schema_id = info->module_name;
+
+  schema = g_settings_schema_source_lookup (info->schema_source, schema_id,
+                                            FALSE);
+
+  if (schema == NULL)
+    return NULL;
+
+  settings = g_settings_new_full (schema, NULL, NULL);
+
+  g_settings_schema_unref (schema);
+
+  return settings;
+}
+
 /**
  * peas_plugin_info_get_dependencies:
  * @info: A #PeasPluginInfo.
