@@ -35,16 +35,14 @@ G_BEGIN_DECLS
 
 typedef struct _TestingCallablePlugin         TestingCallablePlugin;
 typedef struct _TestingCallablePluginClass    TestingCallablePluginClass;
+typedef struct _TestingCallablePluginPrivate  TestingCallablePluginPrivate;
 
 struct _TestingCallablePlugin {
-  /* Inherit from GObject and not PeasExtensionBase
-   * to check that it is possible
-   */
-  GObject parent_instance;
+  PeasExtensionBase parent_instance;
 };
 
 struct _TestingCallablePluginClass {
-  GObjectClass parent_class;
+  PeasExtensionBaseClass parent_class;
 };
 
 GType testing_callable_plugin_get_type (void) G_GNUC_CONST;
