@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -129,6 +129,8 @@ testing_util_init (void)
 
   g_log_set_default_handler (log_handler, NULL);
 
+  g_setenv ("GSETTINGS_BACKEND", "memory", TRUE);
+
   g_irepository_prepend_search_path (BUILDDIR "/libpeas");
 
   g_setenv ("PEAS_PLUGIN_LOADERS_DIR", BUILDDIR "/loaders", TRUE);
@@ -222,7 +224,7 @@ testing_util_pop_log_hook (void)
 
   g_return_if_fail (log_hooks != NULL);
   g_return_if_fail (log_hooks->len > 0);
-  
+
   hook = g_ptr_array_index (log_hooks, log_hooks->len - 1);
 
   if (!hook->hit)
@@ -255,11 +257,19 @@ testing_util_pop_log_hooks (void)
 
   if (unhit_hooks->len == 1)
     {
-      g_error ("Log hook was not triggered: '%s'", hook->pattern);
+      gchar *msg;
+
+      hook = g_ptr_array_index (unhit_hooks, 0);
+      msg = g_strdup_printf ("Log hook was not triggered: '%s'",
+                             hook->pattern);
+
+      /* Use the default log handler directly to avoid recurse complaints */
+      g_log_default_handler (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR, msg, NULL);
+      abort ();
     }
   else if (unhit_hooks->len > 1)
     {
-      /* Avoid our log handler */
+      /* Use the default log handler directly to avoid recurse complaints */
       g_log_default_handler (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR,
                              "Log hooks were not triggered:\n", NULL);
 
